/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.lang.ExceptionInInitializerError;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

// SAX imports
import org.xml.sax.SAXException;

/**
 * <p>
 *  <code>ZeusException</code> is the top level
 *   <code>Exception</code> that Zeus classes
 *   can throw.  It's subclasses add specificity to the 
 *   problems that can occur using Zeus, but this single
 *   <code>Exception</code> can be caught to handle all
 *   Zeus specific problems.
 * </p>
 * 
 * @author Brett McLaughlin
 */
public class ZeusException extends Exception {

    /** A wrapped <code>Throwable</code> */
    protected Throwable cause;

    /**
     * <p>
     *  This will create an <code>Exception</code>.
     * </p>
     */
    public ZeusException() {
        super("An exception occurred in Zeus processing.");
    }

    /**
     * <p>
     *  This will create an <code>Exception</code> with the given message.
     * </p>
     *
     * @param message <code>String</code> message indicating
     *                the problem that occurred.
     */    
    public ZeusException(String message)  {
        super(message);
    }

    /**
     * <p>
     *  This will create an <code>Exception</code> with the given message
     *    and wrap another <code>Exception</code>.  This is useful when
     *    the originating <code>Exception</code> should be held on to.
     * </p>
     *
     * @param message <code>String</code> message indicating
     *                the problem that occurred.
     * @param cause <code>Throwable</code> that caused this
     *                  to be thrown.
     */    
    public ZeusException(String message, Throwable cause)  {
        super(message);    
        this.cause = cause;
    }    

    /** 
     * <p>
     *  Intializes the cause of this exception to be the specified value.
     * </p>
     * @param cause <code>Throwable</code> that caused this
     *                  to be thrown.
     */
    public Throwable initCause(Throwable cause)  {
        this.cause = cause;
        return cause;
    }    

    /**
     * <p>
     * This returns the message for the <code>Exception</code>. If
     *   there are one or more nested exceptions, their messages
     *   are appended.
     * </p>
     *
     * @return <code>String</code> - message for <code>Exception</code>.
     */
    public String getMessage() {
        // Get this exception's message.
        String msg = super.getMessage();

        Throwable parent = this;
        Throwable child;
        
        // Look for nested exceptions.
        while((child = getNestedException(parent)) != null) {
            // Get the child's message.
            String msg2 = child.getMessage();
            
            /**
             * Special case: If a SAXException has no message of its own, 
             *   but has a nested exception, then it returns the nested 
             *   exception's message as its message. We don't want to add 
             *   that message twice.
             */
            if (child instanceof SAXException) {
                Throwable grandchild = ((SAXException)child).getException();
                /**
                 * If the SAXException tells us that it's message is identical 
                 *   to its nested exception's message, then we skip it, so 
                 *   we don't add it twice.
                 */
                if ((grandchild != null) && 
                    (msg2 != null) && 
                    (msg2.equals(grandchild.getMessage()))) {
                    msg2 = null;
                }
            }

            // If we found a message for the child exception, we append it.
            if (msg2 != null) {
                if (msg != null) {
                    msg += ": " + msg2;
                } else {
                    msg = msg2;
                }
            }

            /**
             * Any nested ZeusException will append its own children,
             *  so we need to break out of here.
             */
            if (child instanceof ZeusException) {
                break;
            }
            parent = child;
        }

        // Return the completed message.
        return msg;
    }

    /**
     * <p>
     * This prints the stack trace of the <code>Exception</code>. If
     *   there is a root cause, the stack trace of the root
     *   <code>Exception</code> is printed right after.
     * </p>
     */
    public void printStackTrace() {
        // Print the stack trace for this exception.
        super.printStackTrace();

        Throwable parent = this;
        Throwable child;

        // Print the stack trace for each nested exception.
        while((child = getNestedException(parent)) != null) {
            if (child != null) {
                System.err.print("Caused by: ");
                child.printStackTrace();
                /**
                 * Any nested ZeusException will print its own children,
                 *  so we need to break out of here.
                 */
                if (child instanceof ZeusException) {
                    break;
                }
                parent = child;
            }
        }
    }

    /**
     * <p>
     * This prints the stack trace of the <code>Exception</code> to the given
     *   PrintStream. If there is a root cause, the stack trace of the root
     *   <code>Exception</code> is printed right after.
     * </p>
     */
    public void printStackTrace(PrintStream s) {
        // Print the stack trace for this exception.
        super.printStackTrace(s);

        Throwable parent = this;
        Throwable child;

        // Print the stack trace for each nested exception.
        while((child = getNestedException(parent)) != null) {
            if (child != null) {
                System.err.print("Caused by: ");
                child.printStackTrace(s);
                /**
                 * Any nested ZeusException will print its own children,
                 *   so we need to break out of here.
                 */
                if (child instanceof ZeusException) {
                    break;
                }
                parent = child;
            }
        }
    }

    /**
     * <p>
     * This prints the stack trace of the <code>Exception</code> to the given
     *   PrintWriter. If there is a root cause, the stack trace of the root
     *   <code>Exception</code> is printed right after.
     * </p>
     */
    public void printStackTrace(PrintWriter w) {
        // Print the stack trace for this exception.
        super.printStackTrace(w);

        Throwable parent = this;
        Throwable child;

        // Print the stack trace for each nested exception.
        while((child = getNestedException(parent)) != null) {
            if (child != null) {
                System.err.print("Caused by: ");
                child.printStackTrace(w);
                /**
                 * Any nested JDOMException will print its own children,
                 *  so we need to break out of here.
                 */
                if (child instanceof ZeusException) {
                    break;
                }
                parent = child;
            }
        }
    }

    /**
     * <p>
     * This will return the root cause <code>Throwable</code>, or null
     *   if one does not exist.
     * </p>
     * 
     * @return <code>Throwable</code> - the wrapped <code>Throwable</code>.
     */
    public Throwable getCause()  {
        return cause;             
    }

    /**
     * <p>
     *  This returns any nested exceptions from the supplied 
     *    <code>Throwable</code> object. If there is no nested exception,
     *    then <code>null</code> is returned.
     * </p>
     *
     * @param parent <code>Throwable</code> to check for nested exceptions
     * @return <code>Throwable</code> - the nested exception, or null
     */
    private static Throwable getNestedException(Throwable parent) {
        if (parent instanceof ZeusException) {
            return ((ZeusException)parent).getCause();
        }
        
        if (parent instanceof SAXException) {
            return ((SAXException)parent).getException();
        }
        
        if (parent instanceof InvocationTargetException) {
            return ((InvocationTargetException)parent).getTargetException();
        }
        
        if (parent instanceof ExceptionInInitializerError) {
            return ((ExceptionInInitializerError)parent).getException();
        }
        
        if (parent instanceof RemoteException) {
            return ((RemoteException)parent).detail;
        }
        
        /**
         * These classes are not part of standard JDK 1.1 or 1.2, so we 
         *   use reflection to access them.
         */
        Throwable nestedException = getNestedException(parent, 
            "javax.naming.NamingException", "getRootCause");
        if (nestedException != null) {
            return nestedException;
        }

        return null;
    }

    /**
     * <p>
     *  This method uses reflection to obtain the nested exception of a 
     *    <code>Throwable</code>. We use reflection because the desired class 
     *    may not exist in the currently-running VM.
     * </p>
     *
     * @param parent <code>Throwable</code> to check for nested exceptions
     * @param className the class of the exception to look for
     * @param methodName the method to check for a root cause with
     * @return <code>Throwable</code> - the nested exception, or null
     */
    private static Throwable getNestedException(Throwable parent, 
                                                String className, 
                                                String methodName) {
        try {
            /**
             * See if this Throwable is of the desired type, 
             *   by using isAssignableFrom().
             */
            Class testClass = Class.forName(className);
            Class objectClass = parent.getClass();
            if (testClass.isAssignableFrom(objectClass)) {
                // Use reflection to call the specified method.
                Class[] argClasses = new Class[0];
                Method method = testClass.getMethod(methodName, argClasses);
                Object[] args = new Object[0];
                return (Throwable)method.invoke(parent, args);
            }
        }
        catch(Exception ex) {
            /**
             * Most likely, the desired class is not available in this VM. 
             *   That's fine. Even if it's caused by something else, we 
             *   don't want to display an error here, since we're already 
             *   in the process of trying to display the original
             *   error - another error here will just confuse things.
             */
        }

        return null;
    }
}
