/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

// Zeus imports
import java.util.BitSet;
import java.util.Vector;

/**
 * <p>
 *  <code>AtomicProperty</code> implements the 
 *    <code>{@link org.enhydra.zeus.Binding}</code> and
 *    <code>{@link Property}</code> interfaces, and provides
 *    for a simple name/value property representation.
 * </p><p>
 *  An <code>AtomicProperty</code> will most commonly represent
 *    simple typed values in Java, such as a <code>String</code>,
 *    <code>int</code>, or <code>Date</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Steve Witten
 * @author Maciej Zawadzki
 * @author Sean Ogle
 */
public class AtomicProperty extends BaseProperty {
    
    /** Default modifier for properties */
    private static BitSet DEFAULT_MODIFIER = new BitSet();
    
    static {
        DEFAULT_MODIFIER.set(Property.ACCESS_PRIVATE);
    }

    /**
     * <p>
     *  This will create a new <code>AtomicProperty</code> with
     *    the specified information.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.     
     * @param xmlNamespaceURI the XML namespace URI associated with the 
     *        property's XML name.
     * @param xmlType the XML type associated with the property.
     * @param xmlTypeNamespaceURI the XML namespace URI associated with the 
     *        property's XML type.     
     * @param modifier the <code>int<code> constant for the
     *        property's access modifier.
     * @param enumeration the vector of possible values for this property.
     * @param defaultValue the default value for this property.
     *
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     * @see <code>{@link Property#STORAGE_STATIC}</code>
     * @see <code>{@link Property#MUTABILITY_VOLATILE}</code>
     * @see <code>{@link Property#MUTABILITY_FINAL}</code>          
     */
    public AtomicProperty(String xmlName,
                          String xmlNamespaceURI,
                          String xmlType,
                          String xmlTypeNamespaceURI,
                          BitSet modifier,
                          Vector enumeration,
                          Object defaultValue) {                              
        super();

        // Error checking        
        if (xmlName == null) {
            throw new IllegalArgumentException("An AtomicProperty cannot " +
                "have a null XML name.");
        }
        if (xmlNamespaceURI == null) {
            throw new IllegalArgumentException("An AtomicProperty cannot " +
                "have a null XML namespace URI. To specify no namespace URI, " +
                "use the empty String (\"\")");
        }
        if (xmlType == null) {
            throw new IllegalArgumentException("An AtomicProperty cannot " +
                "have a null XML type.");
        }
        if (xmlTypeNamespaceURI == null) {
            throw new IllegalArgumentException("An AtomicProperty cannot " +
                "have a null XML type namespace URI. To specify no namespace " +
                "URI, use the empty String (\"\")");
        }
        if (modifier == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null set of modifiers.");
        }
        
        this.xmlName = xmlName;
        this.xmlNamespaceURI = xmlNamespaceURI;
        this.xmlType = xmlType;       
        this.xmlTypeNamespaceURI = xmlTypeNamespaceURI;
        this.modifier = modifier;
        this.enumeration = enumeration;
        this.defaultValue = defaultValue;
    }

    /**
     * <p>
     *  This will create a new AtomicProperty with the specified information. 
     *    The modifier will be set to ACCESS_PRIVATE, the default value and the
     *    enumeration will be set to null.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.
     * @param xmlNamespaceURI the XML namespace URI associated with the 
     *        property's XML name.
     * @param xmlType the XML type associated with the property.
     * @param xmlTypeNamespaceURI the XML namespace URI associated with the 
     *        property's XML type.
     *
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     * @see <code>{@link Property#STORAGE_STATIC}</code>
     * @see <code>{@link Property#MUTABILITY_VOLATILE}</code>
     * @see <code>{@link Property#MUTABILITY_FINAL}</code>          
     */
    public AtomicProperty(String xmlName,
                          String xmlNamespaceURI,
                          String xmlType,
                          String xmlTypeNamespaceURI) {
        this(xmlName, xmlNamespaceURI, xmlType, xmlTypeNamespaceURI, 
             DEFAULT_MODIFIER, null, null);
    }
    
    /**
     * <p>
     *  This will create a new AtomicProperty with the specified information. 
     *    The modifier will be set to ACCESS_PRIVATE, the default value and the
     *    enumeration will be set to null, and both the XML name and XML type
     *    namespaces are set to an empty string, the equivalent of no namespace.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.
     * @param xmlType the XML type associated with the property.
     *
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     * @see <code>{@link Property#STORAGE_STATIC}</code>
     * @see <code>{@link Property#MUTABILITY_VOLATILE}</code>
     * @see <code>{@link Property#MUTABILITY_FINAL}</code>          
     */
    public AtomicProperty(String xmlName,
                          String xmlType) {
        this(xmlName, "", xmlType, "",
             DEFAULT_MODIFIER, null, null);
    }    
}
