/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.transform;

/**
 * <p>
 *  <code>TypeConversion</code> is used to specify type conversions used in
 *    <code>{@link org.enhydra.zeus.Transformer}</code> and related classes
 *    and processes. It allows character-based data, as found in XML documents,
 *    to be converted to type-specific data in Java.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class TypeConversion {
    
    /** The name of this type conversion */
    private String name;
    
    /** The Java type to convert to; generally a fully-qualified Java class */
    private String type;
    
    /** The class/method to use for parsing data into this format */
    private String parseMethod;
    
    /** The class/method to use for printing data in this format */
    private String printMethod;
    
    /**
     * <p>
     *  This creates a new <code>Conversion</code> with the supplied
     *    name and type. In most cases, the type specified here should 
     *    be a fully-qualified Java class name, such as "java.util.Date" or 
     *    "java.math.BigDecimal". If the class is in an automatically imported 
     *    class, like "java.lang.Integer", then the package does not need to 
     *    be supplied.
     * </p>
     *
     * @param name the name for this type conversion.
     * @param type the Java type for this type conversion.
     */
    public TypeConversion(String name, String type) {
        this(name, type, null, null);
    }
    
    /**
     * <p>
     *  This creates a new <code>Conversion</code> with the supplied
     *    name and type. In most cases, the type specified here should 
     *    be a fully-qualified Java class name, such as "java.util.Date" or 
     *    "java.math.BigDecimal". If the class is in an automatically imported 
     *    class, like "java.lang.Integer", then the package does not need to 
     *    be supplied. It also allows specification of the method to use
     *    for conversion of data from XML to Java, and for printing Java in
     *    the format this <code>TypeConversion</code> represents.
     * </p>
     *
     * @param name the name for this type conversion.
     * @param type the Java type for this type conversion.
     * @param parseMethod the method to use for parsing data.
     * @param printMethof the method to use for printing data.
     */
    public TypeConversion(String name, String type, 
                          String parseMethod, String printMethod) {
             
        // Error checking
        if (name == null) {
            throw new IllegalArgumentException("A TypeConversion cannot " +
                "have a null name.");
        }
        if (type == null) {
            throw new IllegalArgumentException("A TypeConversion cannot " +
                "have a null type.");
        }
        
        this.name = name;
        this.type = type;
        this.parseMethod = parseMethod;
        this.printMethod = printMethod;
    }
    
    /**
     * <p>
     *  This will set the name of this <code>TypeConversion</code>.
     * </p>
     *
     * @param name the name for this type conversion
     */
    public void setName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("A TypeConversion cannot " +
                "have a null name.");
        }
        this.name = name;
    }
    
    /**
     * <p>
     *  This will return the current name of this <code>TypeConversion</code>.
     * </p>
     *
     * @return <code>String</code> - the name of this type conversion.
     */
    public String getName() {
        return name;
    }
    
    /**
     * <p>
     *  This will set the Java type for data using this <code>Conversion</code>.
     *    In most cases, the type specified here should be a fully-qualified
     *    Java class name, such as "java.util.Date" or "java.math.BigDecimal".
     *    If the class is in an automatically imported class, like
     *    "java.lang.Integer", then the package does not need to be supplied.
     * </p>
     *
     * @param type the Java type for this conversion.
     */
    public void setType(String type) {
        if (type == null) {
            throw new IllegalArgumentException("A TypeConversion cannot " +
                "have a null type.");
        }
        
        this.type = type;
    }
    
    /**
     * <p>
     *  This returns the Java type for this <code>Conversion</code>. This is
     *    generally a fully-qualified Java class, such as "java.util.Date" or
     *    "java.math.BigDecimal".
     * </p>
     *
     * @return <code>String</code> - Java type for this conversion.
     */
    public String getType() {
        return type;
    }
    
    /**
     * <p>
     *  This sets the name of the method to use for conversion from a
     *    character-based piece of XML data into the Java type for this
     *    <code>TypeConversion</code>.
     * </p>
     *
     * @param parseMethod the name of the method to use for parsing data
     */
    public void setParseMethod(String parseMethod) {
        this.parseMethod = parseMethod;
    }
    
    /**
     * <p>
     *  This returns the name of the method to use for conversion from
     *    a character-based piece of XML data into the Java type for this
     *    <code>TypeConversion</code>.
     * </p>
     *
     * @return <code>String</code> - the name of the method to use for parsing.
     */
    public String getParseMethod() {
        return parseMethod;
    }
    
    /**
     * <p>
     *  This sets the name of the method to use for printing data in the
     *    format in this <code>TypeConversion</code>.
     * </p>
     *
     * @param printMethod the name of the method to use for printing data
     */
    public void setPrintMethod(String printMethod) {
        this.printMethod = printMethod;
    }
    
    /**
     * <p>
     *  This returns the name of the method to use for printing data in the
     *    format in this <code>TypeConversion</code>.
     * </p>
     *
     * @return <code>String</code> - the name of the method to use for printing.
     */
    public String getPrintMethod() {
        return printMethod;
    }
}
