/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.transform;

import java.util.LinkedList;
import java.util.List;

/**
 * <p>
 *  <code>ValueEnumeration</code> represents a set of allowed values for a
 *    class or other construct to be allowed to take on. Even though the values
 *    may ultimately be represented by Java <code>int</code>s,
 *    <code>Date</code>s, or other non-character-based types, all data in this
 *    class is represented by Java <code>String</code>s, and later converted
 *    to appropriate types.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class ValueEnumeration {
    
    /** The name of this enumeration */
    private String name;
    
    /** The list of allowed values */
    private List allowedValues;

    /**
     * <p>
     *  Default constructor.
     * </p>
     *
     * @param name the name of this enumeration.
     */
    public ValueEnumeration(String name) {
        this(name, new LinkedList());
    }
    
    /**
     * <p>
     *  This constructor creates a new <code>ValueEnumeration</code> using
     *    the supplied <code>List</code> to seed the allowed values.
     * </p>
     *
     * @param name the name of this enumeration.
     * @param allowedValue initial values to allow for this enumeration.
     */
    public ValueEnumeration(String name, List allowedValues) {
        // Error Checking
        if (name == null) {
            throw new IllegalArgumentException("A ValueEnumeration cannot " +
                "have a null name.");
        }
        
        this.name = name;
        this.allowedValues = allowedValues;
    }
    
    /**
     * <p>
     *  This will set the name of this <code>ValueEnumeration</code>.
     * </p>
     *
     * @param name the name for this enumeration
     */
    public void setName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("A ValueEnumeration cannot " +
                "have a null name.");
        }
        this.name = name;
    }
    
    /**
     * <p>
     *  This will return the current name of this <code>ValueEnumeration</code>.
     * </p>
     *
     * @return <code>String</code> - the name of this enumeration.
     */
    public String getName() {
        return name;
    }
    
    /**
     * <p>
     *  This sets the list of allowed values to the supplied <code>List</code>.
     *    Any existing allowed values are replaced. If any value is allowed,
     *    the <code>null</code> value should be supplied here.
     * </p>
     *
     * @param allowedValues the values to allow for this enumeration.
     */
    public void setAllowedValues(List allowedValues) {
        this.allowedValues = allowedValues;
    }
    
    /**
     * <p>
     *  This returns the current list of allowed values.
     * </p>
     *
     * @return <code>List</code> - the list of allowed values.
     */
    public List getAllowedValues() {
        return allowedValues;
    }
    
    /**
     * <p>
     *  This will add a new allowed value to the current list.
     * </p>
     *
     * @param allowedValue the new value to allow
     */
    public void addAllowedValue(String allowedValue) {
        if (allowedValue == null) {
            throw new IllegalArgumentException("A ValueEnumeration cannot " +
                "have the null value in its set of allowed values. To allow " +
                "a variable to have the null value, use the String \"null\".");
        }
        allowedValues.add(allowedValue);
    }

    /**
     * <p>
     *  This will remove the specified value from the current list.
     * </p>
     *
     * @param removedValue the value to remove from the enumeration.
     */
    public void removeAllowedValue(String removedValue) {
        allowedValues.remove(removedValue);
    }
    
    /**
     * <p>
     *  This will indicate if the supplied value is allowed for this
     *    <code>ValueEnumeration</code>.
     * </p><p>
     *  Note that this operation is <i>case-sensitive</i>; while this is not
     *    relevant if the data is numerical or non-character-based, it will
     *    affect this operation on character-based values.
     * </p>
     *
     * @param value the value to check for legality in this enumeration.
     * @return <code>boolean</code> - whether the supplied value is allowed.
     */
    public boolean isAllowedValue(String value) {
        return allowedValues.contains(value);
    }
}
