/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Iterator;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.Transformer;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.generator.SimpleGenerator;
import org.enhydra.zeus.transform.DefaultsTransformer;

/** 
 * <p>
 *  This provides basic functionality for classes that need to
 *    implement the <code>{@link SourceGenerator}</code> interface.
 * </p>
 *
 * @author Brett McLaughlin
 */
public abstract class BaseSourceGenerator {

    /** The reader for reading constraints from */
    protected Reader constraintsReader;

    /** The directory to output classes to */
    protected File outputDir;

    /** The package to generate classes within */
    protected String javaPackage;
    
    /** Whether or not to collapse simple elements */
    protected boolean collapseSimpleElements;
    
    /** Whether or not to ignore ID attributes */
    protected boolean ignoreIDAttributes;

    /**
     * <p>
     *  This sets up defaults for the generator.
     * </p>
     */
    public BaseSourceGenerator() {
        javaPackage = "";
        outputDir = new File(".");
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>String</code> URI.
     * </p>
     *
     * @param fileURI <code>String</code> URI for constraints file.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(String fileURI) throws IOException {
        if (fileURI == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null file URI.");
        }
        
        FileReader reader = new FileReader(new File(fileURI));
        setConstraintsInput(reader);
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>File</code>.
     * </p>
     *
     * @param file <code>File</code> to read constraints from.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(File file) throws IOException {
        if (file == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null File.");
        }
        
        setConstraintsInput(new FileReader(file));
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>FileReader</code>.
     * </p>
     *
     * @param reader <code>Reader</code> to read constraints
     *               from.
     */
    public void setConstraintsInput(Reader reader) {
        if (reader == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null Reader.");
        }
        constraintsReader = reader;
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as an 
     *    <code>InputStream</code>.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read
     *        constraints from.
     */
    public void setConstraintsInput(InputStream inputStream) {
        if (inputStream == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null InputStream.");
        }
        setConstraintsInput(new InputStreamReader(inputStream));
    }

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDirString <code>String</code> specifying output 
     *        directory for generated classes.
     * @throws <code>IOException</code> - when invalid direcotory is
     *         specified
     */
    public void setOutputDir(String outputDir) throws IOException {
        if (outputDir == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null output directory.");
        }
        setOutputDir(new File(outputDir));
    }

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDir <code>File</code> specifying output directory
     *        for generated classes.
     * @throws <code>IOException</code> - when invalid direcotory is
     *         specified
     */
    public void setOutputDir(File outputDir) throws IOException {
        if (outputDir == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null output directory.");
        }
        
        if ((!outputDir.exists()) || (!outputDir.isDirectory())) {
            throw new IOException("Output directory must exist before " +
                                  "class generation.");
        }
        this.outputDir = outputDir;
    }

    /**
     * <p>
     *  This will set the package for generating classes within.
     * </p>
     *
     * @param javaPackage the package to generate classes within.
     */
    public void setJavaPackage(String javaPackage) {
        if (javaPackage == null) {
            throw new IllegalArgumentException("A Source Generator cannot " +
                "have a null Java package.");
        }
        
        this.javaPackage = javaPackage;
    }
    
    /**
     * <p>
     *  This sets whether or not to collapse simple elements. By default,
     *    simple elements are <i>not</i> collapsed. By default, ID attributes
     *    are <i>not</i> ignored in this determination.
     * </p>
     *
     * @param collapseSimpleElements whether or not to collapse simple
     *        elements.
     */
    public void setCollapseSimpleElements(boolean collapseSimpleElements) {
        setCollapseSimpleElements(collapseSimpleElements, false);
    }
    
    /**
     * <p>
     *  This sets whether or not to collapse simple elements. By default,
     *    simple elements are <i>not</i> collapsed. It also allows specification
     *    of whether ID attributes should be ignored when making a determination
     *    if an element is simple.
     * </p>
     *
     * @param collapseSimpleElements whether or not to collapse simple
     *        elements.
     * @param ignoreIDAttributes whether or not to ignore ID attributes.
     */
    public void setCollapseSimpleElements(boolean collapseSimpleElements,
                                          boolean ignoreIDAttributes) {
    
        this.collapseSimpleElements = collapseSimpleElements;
        this.ignoreIDAttributes = ignoreIDAttributes;
    }

    /**
     * <p>
     *  This method performs class generation.
     * </p>
     *
     * @throws <code>IOException</code> - when class generation fails
     * @throws <code>ZeusException</code> - when class generation fails.
     */
    public void generate() throws IOException, ZeusException {    
        // Transformer for bindings
        Transformer transformer = new DefaultsTransformer();
        transformer.getTransformerOptions().setDefaultPackage(javaPackage);
        
        // Generator for source code
        Generator generator = getGenerator();
        generator.setOutputDirectory(outputDir);

        // Get the DTD bindings
        List bindings = getConstraintBindings();
        
        // Transform the bindings
        List transformedBindings = transformer.transform(bindings);
        
        // Generate source code
        for (Iterator i = transformedBindings.iterator(); i.hasNext(); ) {
            Binding binding = (Binding)i.next();
            generator.generate(binding);
        }
    }

    /**
     * <p>
     *  This defines a contract for subclasses to allow them to get the
     *    Zeus <code>{@link Binding}</code> objects to generate code
     *    from.
     * </p>
     *
     * @return <code>List</code> - a list of the <code>{@link Binding}</code>
     *         objects to generate code from.
     * @throws <code>IOException</code> - when binding creation fails
     */
    protected abstract List getConstraintBindings() throws IOException;
    
    /**
     * <p>
     *  This handles the process of creating a <code>{@link Generator}</code>
     *    for use in data binding. 
     * </p><p>
     *  This version returns a basic
     *    <code>{@link SimpleGenerator}</code>, but other <code>Generator</code>
     *    implementations can be used by overriding this method.
     * </p>
     *
     * @return <code>Generator</code> - the <code>Generator</code> to use.
     */
    protected Generator getGenerator() {
        return new SimpleGenerator();
    }
}
