/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Properties;

// Ant imports
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

// Reflection
import java.lang.reflect.*;

/**
 * <p>
 *  <code>ZeusTask</code> is an Ant <code>Task</code> for generating Java
 *   source files from a set of constraints. Please refer to the 
 *   <a href="http://jakarta.apache.org/ant/index.html">Ant website</a>
 *   for more information. The supported tags are as follows:
 * </p>
 * <p>
 *  The <code>zeus</code> element is the top-level element used to initiate
 *    data binding using Zeus. It specifies global options for all constraint
 *    generation processes initiated within it (each using the <code>constraint</code>
 *    element, see below).<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>&lt;zeus&gt;</b> Tag<br />
 *  <table border="1" cellspacing="3" cellpadding="3" width="100%">
 *    <tr>
 *      <th>attribute</th>
 *      <th>required</th>
 *      <th>default value</th>
 *      <th>description</th>
 *    </tr>
 *    <tr>
 *      <td>srcDir</td>
 *      <td>no</td>
 *      <td>[Current Working Directory]</td>
 *      <td>The base directory for constraint files</td>
 *    </tr>
 *    <tr>
 *      <td>destDir</td>
 *      <td>yes</td>
 *      <td><i>N/A</i></td>
 *      <td>The directory in which generated Java source files should be
 *        placed. Note that this will be the <i>base</i> directory, and
 *        directories equivalent to the package-level of the generated
 *        source code will be nested within this base.
 *      </td>
 *    </tr>
 *    <tr>
 *      <td>defaultJavaPackage</td>
 *      <td>no</td>
 *      <td>""</td>
 *      <td>The Java package to use on all constraint generation processes
 *        where an explicit Java package is not supplied (using the
 *        <code>javaPackage</code> attribute on the <code>constraint</code>
 *        element, see below).</td>
 *    </tr>
 *  </table>
 * </p><p>
 *  The <code>constraint</code> element can be nested inside the <code>zeus</code>
 *    element, one or more times. This is used to specify a single constraint generation
 *    process (the conversion from one constraint file to one or more Java source
 *    files). All options specified here override the default options specified on
 *    the <code>zeus</code> element.<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>&lt;constraint&gt;</b> Tag<br />
 *  <table border="1" cellspacing="3" cellpadding="3" width="100%">
 *    <tr>
 *      <th>attribute</th>
 *      <th>required</th>
 *      <th>default value</th>
 *      <th>description</th>
 *    </tr>
 *    <tr>
 *      <td>type</td>
 *      <td>yes</td>
 *      <td><i>N/A</i></td>
 *      <td>The type of constraint being supplied. Currently, only <b>DTD</b>
 *        is supported, with <b>XSD</b> (XML Schema) in an alpha state.</td>
 *    </tr>
 *    <tr>
 *      <td>constraintFile</td>
 *      <td>yes</td>
 *      <td><i>N/A</i></td>
 *      <td>The path (working from the <code>sourceDir</code> path, specified
 *        in the <code>zeus</code> element) to the constraint file to generate
 *        source code from.</td>
 *    </tr>
 *    <tr>
 *      <td>javaPackage</td>
 *      <td>no</td>
 *      <td>""</td>
 *      <td>The Java package in which to place generated source code. This overrides
 *        any value supplied in the <code>defaultJavaPackage</code> attribute on
 *        the <code>zeus</code> element.)</td>
 *    </tr>
 *    <tr>
 *      <td>collapseSimpleElements</td>
 *      <td>no</td>
 *      <td>false</td>
 *      <td>Whether or not to collapse simple elements. A simple element is one
 *        in which there is only character content; there are no attributes, and
 *        no nested elements. Collapsing these elements results in properties
 *        that access the simple element's content as <code>get[ElementName]()</code>
 *        instead of <code>get[ElementName]().getValue()</code>.</td>
 *    </tr>
 *    <tr>
 *      <td>ignoreIDAttributes</td>
 *      <td>no</td>
 *      <td>false</td>
 *      <td>Whether or not to ignore ID attributes in deterimining if an element
 *        is simple. When IDREFs are used, most elements have ID attributes that
 *        have no functional meaning. This allows elements with just an ID attribute
 *        to still be collapsed as simple elements.</td>
 *    </tr>
 *  </table>
 * </p>
 *
 * @author J. Matthew Pryor
 * @author Brett McLaughlin
 */
public class ZeusTask extends Task {

    /** The name of the DTD constraint type */
    private static final String CONSTRAINT_TYPE_DTD = "DTD";
    
    /** The name of the DTD constraint type */
    private static final String CONSTRAINT_TYPE_XSD = "XSD";
    
    /** The directory for specified constraints */
    private File sourceDir;
    
    /** The output directory for generated Java source files */
    private File destDir;
    
    /** The default Java package */
    private String defaultJavaPackage;
    
    /** Whether or not simple elements are collapsed in the default case */
    private boolean defaultCollapseSimpleElements;

    /** The constraints used as input for class generation */
    private Vector constraints;


    /** 
     * <p>
     *  This default constructor sets up task defaults.
     * </p>
     */
    public ZeusTask() {
        defaultCollapseSimpleElements = false;
        defaultJavaPackage = "";
        constraints = new Vector();
    }

    /**
     * <p>
     *  This will handle task execution pre-processing tasks. It processes 
     *    these parameters:
     * <ul>
     *  <li>sourceDir: source directory for constraints supplied. If not
     *    specified, this defaults to the current working directory.</li>
     *  <li>destDir: destination directory for the generated source files. If
     *    this does not exist, a <code>BuildException</code> will result.</li>
     * </ul>
     * </p>
     *
     * @throws <code>BuildException</code> - when errors occur in processing the
     *         task.
     */
    public void execute() throws BuildException {
        // See if there is a base source directory specified (sourceDir)
        if (sourceDir == null) {
            sourceDir = project.resolveFile(".");
        }

        // Ensure the destination directory is supplied (destDir)
        if (destDir == null) {
            throw new BuildException("The destDir attribute is required, and " +
                "should indicate the destination directory for generated " +
                "source files.");
        }

        // Process all the constraints marked for execution
        for (Enumeration e = constraints.elements(); e.hasMoreElements(); ) {
            generateFrom((Constraint)e.nextElement());
        }
    }
    
    /**
     * <p>
     *  This will return the Java <code>File</code> object representing the 
     *    source directory for constraints.
     * </p>
     *
     * @return <code>File</code> - the source directory for constraints.
     */
    public File getSourceDir() {
        return sourceDir;
    }

    /**
     * <p>
     *  This sets the source directory to use as a base directory for resolving
     *    specified constraints.
     * </p>
     *
     * @param sourceDir the <code>File</code> representing the source directory.
     */
    public void setSourceDir(File sourceDir) {
        if (!sourceDir.isDirectory()) {
            throw new IllegalArgumentException("The source directory " +
                "must be a valid, existing directory.");
        }
        this.sourceDir = sourceDir;
    }

    /**
     * <p>
     *  This will return the destination directory in which Java source files
     *    are generated.
     * </p>
     *
     * @return <code>File</code> - the output directory.
     */
    public File getDestDir() {
        return destDir;
    }

    /**
     * <p>
     *  This will set the destination directory to use for outputting Java 
     *    source files within.
     * </p>
     *
     * @param destDir <code>File</code> object for destination directory.
     */
    public void setDestDir(File destDir) {
        if (!destDir.isDirectory()) {
            throw new IllegalArgumentException("The destination directory " +
                "must be a valid, existing directory.");
        }
        this.destDir = destDir;
    }

    /**
     * <p>
     *  This will indicate the Java package used for generated source files by
     *    default (if no other package is specified for a specific binding).
     * </p>
     *
     * @return <code>String</code> - the Java package used by default.
     */
    public String getDefaultJavaPackage() {
        return defaultJavaPackage;
    }

    /**
     * <p>
     *  This sets the Java package to use for generated source files in the
     *    default case (unless overridden by a specific generation process).
     * </p>
     *
     * @param defaultJavaPackage the Java package for generated classes.
     */
    public void setDefaultJavaPackage(String defaultJavaPackage) {
        this.defaultJavaPackage = defaultJavaPackage;
    }
    
    /**
     * <p>
     *  This will set whether simple elements or collapsed, in the default
     *    case (when a specific constraint generation does not override it).
     * </p>
     *
     * @param defaultCollapseSimpleElements whether to collapse simple elements.
     */
    public void setDefaultCollapseSimpleElements(
        boolean defaultCollapseSimpleElements ) {
            
        this.defaultCollapseSimpleElements = defaultCollapseSimpleElements;
    }

    /**
     * <p>
     *  This will create a new Ant <code>Constraint</code>, add it to the list
     *    of Ant constraints, and return the newly created 
     *    <code>Constraint</code>.
     * </p>
     *
     * @return <code>Constraint</code> - the created constraint.
     */
    public Constraint createConstraint() {
        Constraint constraint = new Constraint();
        constraints.addElement(constraint);
        return constraint;
    }

    /**
     * <Please write something about this operation>
     * @param constraint <Please write something about this parameter>
     * @exception BuildException <Please write something about why this exception would be thrown>
     */
    public void generateFrom(Constraint constraint) throws BuildException {
        if (constraint.getType().equalsIgnoreCase(CONSTRAINT_TYPE_DTD)) {
            try {
                String[] args = null;
                int i = 0;

                // Determine if a Java package is present
                if (constraint.getJavaPackage() != null) {
                    args = new String[6];
                } else {
                    args = new String[5];
                }

                // Constraint to process
                String filename = sourceDir.getAbsolutePath() + File.separator +
                    constraint.getConstraintFile().getName();
                args[i++] = "-constraints=" + filename;
                    
                // Output directory
                args[i++] = "-outputDir=" + destDir.getAbsolutePath();
                
                // Whether or not to ignore ID attributes
                args[i++] = "-ignoreIDAttributes=" + 
                    constraint.ignoreIDAttributes();
                
                // Whether or not to collapse simple elements
                args[i++] = "-collapseSimpleElements=" + 
                    constraint.collapseSimpleElements();
                
                // Always use the "simple" generator
                args[i++] = "-generator=simple";
                
                // The Java Package name to use
                if (constraint.getJavaPackage() != null) {
                    args[i++] = "-javaPackage=" + constraint.getJavaPackage();
                }
                
                System.out.println("Generating Java from DTD: " + filename);
                System.out.println("  + Output Directory: " + 
                    destDir.getAbsolutePath());
                System.out.println("  + Java Package: " + 
                    constraint.getJavaPackage());
                if (constraint.collapseSimpleElements()) {
                    System.out.println("  + Collapsing Simple Elements");
                }
                if (constraint.ignoreIDAttributes()) {
                    System.out.println("  + Ignoring ID Attributes");
                }
                System.out.println("\n");

                // Invoke the generator with supplied arguments
                String generatorClassName = "org.enhydra.zeus.util.DTDSourceGenerator";
                Class c = null;
                Method m = null;
                try {
                    c = Class.forName(generatorClassName);
                    m = c.getMethod("main", new Class[] { String[].class });
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new BuildException(e);
                }
                
                if (m != null) {
                    m.invoke(null, new Object[] { args });
                }
            } catch (Exception e) {
                e.printStackTrace();
                throw new BuildException(e);
            }
        } else {
            throw new BuildException("The specified constraint type, '" +
                constraint.getType() + "', is not currently supported.");
        }
    }

    /**
     * <p>
     *  <code>Constraint</code> is an inner class used to represent a single
     *    constraint element (<code>constraint</code>). It determines all
     *    of the attributes specified on that element.
     * </p>
     */
    public class Constraint {
        
        /** The constraint to use for input */
        private File constraintFile;
        
        /** The constraint type */
        private String type;
        
        /** The Java package for the generated source */
        private String javaPackage;
        
        /** Whether or not to ignore ID attributes */
        private boolean ignoreIDAttributes;
        
        /** Whether or not to collapse simpleElements */
        private boolean collapseSimpleElements;
        
        /**
         * <p>
         *  This constraint sets up default values.
         * </p>
         */
        public Constraint() {
            javaPackage = "";
            ignoreIDAttributes = false;
            collapseSimpleElements = false;
        }
        
        /**
         * <p>
         *  This will return the Java <code>File</code> object representing the
         *    constraint to use for source code generation.
         * </p>
         *
         * @return <code>File</code> - the constraint to generate code from.
         */
        public File getConstraintFile() {
            return constraintFile;
        }

        /**
         * <p>
         *  This will set the file to use as a constraint for source code
         *    generation.
         * </p>
         *
         * @param constraintFile <code>File</code> for XML constraint set.
         */
        public void setConstraintFile(File constraintFile) {
            this.constraintFile = constraintFile;
        }
        
        /**
         * <p>
         *  This will return the type of constraint being processed.
         * </p>
         *
         * @return <code>String</code> - the constraint type.
         */
        public String getType() {
            return type;
        }

        /**
         * <p>
         *  This will set the constraint type of the constraint file supplied.
         * </p>
         *
         * @param type the type of constraint to process.
         */
        public void setType(String type) {
            this.type = type;
        }

        /**
         * <p>
         *  This returns the Java package used in generated classes.
         * </p>
         *
         * @return <code>String</code> - the Java package for generated classes.
         */
        public String getJavaPackage() {
            return javaPackage;
        }
        
        /**
         * <p>
         *  This sets the Java package to use for generated classes.
         * </p>
         *
         * @param javaPackage the package to use for generated classes.
         */
        public void setJavaPackage(String javaPackage) {
            this.javaPackage = javaPackage;
        }
        
        /**
         * <p>
         *  This indicates whether or not ID attributes are ignored in the
         *    constraint generation and determination of simple elements.
         * </p>
         *
         * @return <code>boolean</code> - whether ID attributes are ignored.
         */
        public boolean ignoreIDAttributes() {
            return ignoreIDAttributes;
        }

        /**
         * <p>
         *  This will set whether or not to ignore ID attributes when 
         *    determining if an element is "simple".
         * </p>
         *
         * @param ignoreIDAttributes whether or not to ignore ID attributes.
         */
        public void setIgnoreIDAttributes(boolean ignoreIDAttributes) {
            this.ignoreIDAttributes = ignoreIDAttributes;
        }

        /**
         * <p>
         *  This will indicate whether or not simple elements are collapsed.
         * </p>
         *
         * @return <code>boolean</code> - whether or not simple elements are
         *         collapsed.
         */
        public boolean collapseSimpleElements() {
            return collapseSimpleElements;
        }

        /**
         * <p>
         *  This will set whether or not simple elements are collapsed.
         * </p>
         *
         * @param collapseSimpleElements - whether or not simple elements 
         *        are collapsed.
         */
        public void setCollapseSimpleElements(boolean collapseSimpleElements ) {
            this.collapseSimpleElements = collapseSimpleElements;
        }
    }
}